// Prediction with discrete model (off-line)
// Vary: - length of prediction
//       - uncertainty of the simulated model
//       - imput signal (effect on estimation)
exec("ScIntro.sce",-1),mode(0)

nd=100;
np=5;
th1=[0.8 0.1 0.4 0.7]';            // parameters for simulation (for y=1)
th=[th1 1-th1];                    // all parameters
u=(rand(1,nd)>.3)+1;               // input
y=ones(1,nd);

// simulation
for t=2:nd
  i=2*(u(t)-1)+y(t-1);             // row of the table
  y(t)=(rand(1,1,'unif')>th(i,1))+1; // output generation
end

// estimation
nu=zeros(4,2);
for t=2:nd
  i=2*(u(t)-1)+y(t-1);             // row of the table
  nu(i,y(t))=nu(i,y(t))+1;         // statistics update
end
Eth=nu./(sum(nu,2)*ones(1,2));     // estimate of parameters

// prediction
yj=ones(1,nd);                     // fictitious predicted output
for t=2:(nd-np)
  yj(t)=y(t);                      // at time t start with real output
  for j=1:np
    tj=t+j;                        // future times
    i=2*(u(tj)-1)+yj(tj-1);        // row of the table
    yj(tj)=(rand(1,1,'unif')>Eth(i,1))+1; // prediction generation
  end
  yp(tj)=yj(tj);                   // np-step predction
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

s=(np+3):nd;
plot(s,y(s),'.:',s,yp(s),'rx')
set(gcf(),'position',[300 100 500 400])
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title(string(np)+'-steps ahead prediction')
