// State estimation (vector input and output)
// - simulation: 4-states, 3-inputs and 2-outputs
// - uses function Kalman (recomputation of state statistics)
//     in succession: prediction -> filtration
exec("ScIntro.sce",-1),mode(0), getd()

nd=100;                       // length of simulation/estimation
M=[.3 .5 -.2 .1               // parameters for simulation model 
   .2 .6 .3 -.2
   .4 -.2 .6 .1
   .1 .2 -.4 .8];
N=[1 .2 .5
   .6 1 .2
   .7 .1 1
   .1 .2 .2];
A=[1 0 0 0
   0 1 0 0];
B=[.1 .3 .2
   .5 .3 .8];
rv=.1; rw=.1;
 
nx=max(size(M));
[ny,nu]=size(B);
ut=rand(nu,nd,'n');           // input generation
yt=zeros(ny,nd);
xt=zeros(nx,nd);
x=zeros(nx,1);                // initial state

// SIMULATION
for t=1:nd
  u=ut(:,t);
  x=M*x+N*u+sqrt(rw)*rand(nx,1,'n');    // state generation
  y=A*x+B*u+sqrt(rv)*rand(ny,1,'n');    // output generation
  yt(:,t)=y; xt(:,t)=x;                 // remembering
end

x=zeros(nx,1);                          // initial state estimate
Rv=.01*eye(ny,ny); Rw=.1*eye(nx,nx);    // model covariance matrices      
Rx=1e3*eye(nx,nx);                      // state covariance matrix
xe=zeros(nx,nd); yh=zeros(ny,nd);

// FILTRATION
for t=1:nd
  [x,Rx,yp]=Kalman(x,yt(:,t),ut(:,t),M,N,A,B,Rw,Rv,Rx);
                                        // statistics recomputation
  xe(:,t)=x;                           // remembering
  yh(:,t)=yp;
  ee(:,t)=yt(:,t)-yp;
  rr(:,t)=diag(Rx);
end

// RESULTS
s=1:nd;
// plot of output and predicted output
set(scf(1),'position',[70 70 500 500])
for i=1:ny
  subplot(ny,1,i)
  plot(s,yt(i,s),'b')
  plot(s,yh(i,s),'r')
title(['Output  ',string(i),' and its prediction'])
end
// plot of state and state estimate
set(scf(2),'position',[600 70 500 500])
for i=1:nx
  subplot(nx,1,i),
  plot(s,xt(i,s),'b')
  plot(s,xe(i,s),'r')
  title(['State ',string(i),' and its estimate'])
end
