// Control with scalar 1st order regression model
// - simulated data  y(t)=a*y(t-1)+b*u(t)+k+e(t);
// - state realization of the model for synthesis
// - control on a single control interval with the length nd
// - following a settpoint s(t)
// ----------------------------------------------------------
exec SCIHOME/ScIntro.sce, mode(0)

nd=100;                            // length of control interval
a=.996; b=1; k=-3;                   // regression model parameters
om=1; la=.01;                      // penalization (output, input)
s=sign(100*sin(18*(1:nd)/nd));     // setpoint generation
// conversion to state-space model (M is computed on-line)
N=[b 1 0]'; 
Om=diag([1 la 0]);                 // matrix penalization

S=list();
R=list();
R(nd+1)=zeros(Om);                 // initial condition for dyn. progr.

// CONTROL
// computation of control-law
for t=nd:-1:2
M=[a 0 k-s(t); 0 0 0; 0 0 1];      // state matrix with set-point
  T=R(t+1)+Om;
  A=N'*T*N;
  B=N'*T*M;
  C=M'*T*M;
  S(t)=inv(A)*B;
  R(t)=C-S(t)'*A*S(t);
end

// control-law realization
y(1)=5;
u(1)=0;
for t=2:nd
  u(t)=-S(t)*[y(t-1) u(t-1) 1]';             // optimaal control
  y(t)=a*y(t-1)+b*u(t)+k+.1*rand(1,1,'n');   // simulation
end

// RESULTS
x=1:nd;
plot(x,y(x),'--',x,u(x),x,s(x),':')
legend('y','u','s');
